"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sizeSnapshot = undefined;

var _path = require("path");

var _uglifyEs = require("uglify-es");

var _gzipSize = require("gzip-size");

var _gzipSize2 = _interopRequireDefault(_gzipSize);

var _bytes = require("bytes");

var _bytes2 = _interopRequireDefault(_bytes);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _treeshakeWithRollup = require("./treeshakeWithRollup.js");

var _treeshakeWithWebpack = require("./treeshakeWithWebpack.js");

var _snapshot = require("./snapshot.js");

var snapshot = _interopRequireWildcard(_snapshot);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const validateOptions = options => {
  const optionsKeys = ["snapshotPath", "matchSnapshot", "printInfo"];

  const invalidKeys = Object.keys(options).filter(d => !optionsKeys.includes(d));

  if (invalidKeys.length) {
    throw Error(`Options ${invalidKeys.map(d => `"${d}"`).join(", ")} are invalid`);
  }
};

const bytesConfig = { thousandsSeparator: ",", unitSeparator: " ", unit: "B" };

const formatSize = d => _chalk2.default.bold(_bytes2.default.format(d, bytesConfig));

const sizeSnapshot = exports.sizeSnapshot = (options = {}) => {
  validateOptions(options);

  const snapshotPath = options.snapshotPath || (0, _path.join)(process.cwd(), ".size-snapshot.json");
  const shouldMatchSnapshot = options.matchSnapshot === true;
  const shouldPrintInfo = options.printInfo !== false;

  return {
    name: "size-snapshot",

    transformBundle(source, outputOptions) {
      const format = outputOptions.format;
      const output = outputOptions.file;
      const shouldTreeshake = format === "es";

      if (typeof output !== "string") {
        throw Error("output file in rollup options should be specified");
      }

      const getSize = code => code.length;
      const minified = (0, _uglifyEs.minify)(source).code;
      const treeshakeSize = code => Promise.all([(0, _treeshakeWithRollup.treeshakeWithRollup)(code).then(getSize), (0, _treeshakeWithWebpack.treeshakeWithWebpack)(code).then(getSize)]);

      return Promise.all([(0, _gzipSize2.default)(minified), shouldTreeshake ? treeshakeSize(source) : [0, 0]]).then(([gzippedSize, [rollupSize, webpackSize]]) => {
        const sizes = {
          bundled: getSize(source),
          minified: getSize(minified),
          gzipped: gzippedSize
        };

        let infoString = "\n" + `Computed sizes of "${output}" with "${format}" format\n` + `  bundled: ${formatSize(sizes.bundled)}\n` + `  minified with uglify: ${formatSize(sizes.minified)}\n` + `  minified and gzipped: ${formatSize(sizes.gzipped)}\n`;

        const formatMsg = (msg, size) => {
          return `  ${msg}: ${formatSize(size)}\n`;
        };

        if (shouldTreeshake) {
          sizes.treeshaked = {
            rollup: rollupSize,
            webpack: webpackSize
          };

          infoString += formatMsg("treeshaked with rollup and uglified", rollupSize);
          infoString += formatMsg("treeshaked with webpack in production mode", webpackSize);
        }

        infoString += "\n";

        const snapshotParams = { snapshotPath, name: output, data: sizes };
        if (shouldMatchSnapshot) {
          snapshot.match(snapshotParams);
        } else {
          if (shouldPrintInfo) {
            console.info(infoString);
          }
          snapshot.write(snapshotParams);
        }

        return null;
      });
    }
  };
};